# オルンシュタイン・ウーレンベックモデルの構成・シミュレーション
require(yuima)
# require(cubature) # adaBayes関数で method を nomcmc に設定するとき必要なパッケージですが, 最新バージョンだと動かないことが判明したためコメントアウトしています.

mod.ou <- setModel(drift="-alpha1*(x-alpha2)",diffusion="beta")
n <- 10000 
Terminal <- 5*n^{1/3} 
samp <- setSampling(Initial = 0, Terminal = Terminal, n = n)
ou <- setYuima(mod=mod.ou,sampling = samp)
set.seed(123)
### 初期値
x0 <- 1
### パラメータの真値
param <- list(alpha1 = 3, alpha2 = 1, beta = 0.3)
### シミュレーション
obj <- simulate(ou,xinit=x0,true.parameter=param) 
plot(obj)


# 疑似最尤推定と適応型疑似ベイズ推定
### パラメータの設定
start <- list(alpha1 = 2, alpha2 = 1.3, beta = 1.6) 
lower <- list(alpha1 = 0.1, alpha2 = 0.5, beta = 0.1) 
upper <- list(alpha1 = 5, alpha2 = 2, beta = 2.5)
### 事前分布の設定
prior <- list(alpha1 = list(measure.type = "code",
                            df = "dunif(alpha1, 0.1, 5)"), 
              alpha2 = list(measure.type = "code",
                            df = "dunif(alpha1, 0.5, 2)"),
              beta = list(measure.type = "code",
                          df = "dunif(alpha1, 0.1, 2.5)"))
### 疑似最尤推定量の計算
res <- qmle(obj,start = start, lower = lower,
            upper = upper, joint = TRUE, rcpp = TRUE) 
### 適応型疑似ベイズ推定量の計算
bres <- adaBayes(obj,start = start, lower = lower,
                 upper = upper, rcpp = TRUE, method ="mcmc")

### 結果
summary(res)
summary(bres)

### クラスの詳細
str(res)
str(bres)

### 各項目の抜き出し
res@coef
coef(res)
bres@coef


