require(yuima)
### データの入手
data(LogSPX) 
### データの構造
str(Data) 
### 推定に用いるデータ
x <- Data$logdayprice[1:600] 
### yuima オブジェクトの構成
y <- setYuima(data=setData(x,delta = 1/300))
plot(y)
dev.off()
### パラメトリックモデルの設定
SPX.mod <- setModel(drift = c("mu*x"),
                    diffusion = matrix(c("sigma*x"),1,1)) 
### データの代入
SPX.mod.data <- setYuima(model = SPX.mod, data = y@data)
str(SPX.mod.data) 
### 疑似最尤推定量の計算
SPX.fit <- qmle(SPX.mod.data, lower=list(mu=0.01, sigma=0.01),
                start=list(mu=1, sigma=1),
                upper=list(mu=10, sigma=10), method="L-BFGS-B") 
### 推定値の抜き出し
SPX.fit@coef 
### 予測モデルの構成
SPX.mod.pred <- setYuima(model=SPX.mod, sampling =
                           setSampling(Terminal=0.24, n=240)) 
### 予測パスのシミュレーション
for(i in 1:100){plot(simulate(SPX.mod.pred,
                               true.parameter = list(sigma=SPX.fit@coef[1],
                                                     mu=SPX.fit@coef[2]),
                               xinit = Data$logdayprice[600]), ylim=c(7,8.2),xlim=c(0,0.24), par(new=TRUE), col="green", xlab=c(""), ylab=c(""))
} 
SPX.y <- setYuima(data=setData(Data$logdayprice[600:671],
                               delta=1/300)) 
plot(SPX.y, ylim=c(7,8.2),xlim=c(0,0.24),par(new=TRUE),col="red",lwd=2, main="SPX500")

