require(yuima)
require(quantmod)
getSymbols("IBM", from = "2016-01-01", to = "2017-12-31")
x <- setYuima(data=setData(IBM$IBM.Close)) 
### タイムスケールの設定
y <- setYuima(data=setData(IBM$IBM.Close, delta=1/252))
### 統計モデルの設定
IBM.mod <- setModel(drift=c("mu*x"),diffusion=matrix(c("sigma*x"),1,1))
### データの代入
IBM.mod.data <- setYuima(model=IBM.mod, data=y@data)
### 疑似最尤推定量の計算
IBM.fit <- qmle(IBM.mod.data, lower=list(mu=0.01, sigma=0.01),
                start=list(mu=1, sigma=1), upper=list(mu=10, sigma=10),
                method="L-BFGS-B")
### 疑似最尤推定量の値
IBM.fit@coef

getSymbols("IBM", from = "2018-01-02", to = "2018-04-05")
IBM.first <- IBM$IBM.Close["2018-01-02"] 
set.seed(127)
### 予測モデルの設定
IBM.mod.pred <- setYuima(model=IBM.mod, 
                         sampling = setSampling(Terminal=0.25, n=250))
### パスのシミュレート
IBM.pred <- simulate(IBM.mod.pred,
                     true.parameter = list(sigma=IBM.fit@coef[1],
                                           mu=IBM.fit@coef[2]), 
                     xinit = IBM.first)
plot(IBM.pred, ylim=c(100,220), xlim=c(0,0.27), col="skyblue",
     ylab=c("IBM.Close"),xlab=c("t"))
mypred <- function(param){
  result <- simulate(IBM.mod.pred, 
                     true.parameter = list(sigma=param[1], mu=param[2]), 
                     xinit = IBM.first)
  return(get.zoo.data(result)[[1]])
}
### 100本のサンプルパスを生成
path <- replicate(100, mypred(IBM.fit@coef))
time.idx <- IBM.mod.pred@sampling@grid[[1]] 
matplot(time.idx, path, type = "l", lty = 1, col="skyblue",
        xlab = "t", ylab = "IBM.Close", axes = FALSE,
        main="IBM 2018.01.02-2018.04.04")
IBM2018.y <- setYuima(data=setData(IBM$IBM.Close, delta=1/252))
IBM2018.y <- get.zoo.data(IBM2018.y)[[1]]
### 実データを上書き 
lines(time(IBM2018.y),IBM2018.y, col="red",lwd=2)
idx <- seq(1, length(time(IBM)), by = 9)
axis(1, at = time(IBM2018.y)[idx], labels = time(IBM)[idx])
axis(2)